import { describe, it, expect, vi } from 'vitest';
import { validateInsight } from '../../utils/insights/validateInsight';
import type { InsightModel, InsightRequest } from 'ts-common';
import { log } from '../../logger';
import { Logger } from 'ts-common';

Logger.excludedOwners.push('ts-common');
Logger.excludedOwners.push('query-service');

describe('validateInsight', () => {
  const baseInsight: InsightModel = {
    id: 1,
    name: 'Test Insight',
    description: 'A test insight',
    recipients: ['user@example.com'],
    frequency: 'daily',
    saveStateId: '1c9351d5-0dc5-4c08-979f-829f9f83bb29',
    userId: 42,
    status: true,
    type: 'report',
    template: 'this is an email',
    alarmMode: 'conditional',
    createdAt: '2023-12-12',
    updatedAt: '2023-12-12',
    conditionsCheck: [{ nodeLabel: 'Movie', statistic: 'Count', operator: '>', value: 50 }],
  };

  it('should return true for a valid insight', () => {
    expect(validateInsight(baseInsight, true)).toBe(true);
  });

  it('should return false if insight is null', () => {
    expect(validateInsight(null as unknown as InsightModel, true)).toBe(false);
  });

  it('should return false if required fields are missing', () => {
    const invalidInsight = { ...baseInsight, template: null };
    expect(validateInsight(invalidInsight as unknown as InsightModel, true)).toBe(false);
  });

  it('should return false if alarm mode is disabled and force is false', () => {
    const disabledInsight: InsightModel = {
      ...baseInsight,
      alarmMode: 'disabled',
    };
    expect(validateInsight(disabledInsight, false)).toBe(false);
  });

  it('should return true if alarm mode is disabled but force is true', () => {
    const disabledInsight: InsightModel = { ...baseInsight, alarmMode: 'disabled' };
    expect(validateInsight(disabledInsight, true)).toBe(true);
  });

  it('should return false if no recipients are provided', () => {
    const noRecipientsInsight = { ...baseInsight, recipients: [] };
    expect(validateInsight(noRecipientsInsight, true)).toBe(false);
  });

  it('should return false if userId is null', () => {
    const noUserInsight: InsightModel = { ...baseInsight, userId: null as unknown as number };
    expect(validateInsight(noUserInsight, true)).toBe(false);
  });
});
