import { describe, it, expect } from 'vitest';
import { validateTypeInsight } from '../../utils/insights/validateTypeInsight';
import { Logger } from 'ts-common';
import type { InsightModel, AlarmModes, conditionalOperators } from 'ts-common';

Logger.excludedOwners.push('ts-common');
Logger.excludedOwners.push('query-service');

describe('validateTypeInsight', () => {
  const baseInsight: InsightModel = {
    id: 1,
    name: 'Test Insight',
    description: 'A test insight for validation',
    recipients: [],
    template: 'default',
    conditionsCheck: [],
    frequency: 'daily',
    previousResultHash: 'hash',
    createdAt: '',
    updatedAt: '',
    saveStateId: '',
    type: 'report',
    alarmMode: 'conditional',
  };

  it('should return false when alarmMode is "disabled" and force is false', () => {
    const insight = { ...baseInsight, alarmMode: 'disabled' as AlarmModes };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(false);
  });

  it('should return true when alarmMode is "disabled" and force is true', () => {
    const insight = { ...baseInsight, alarmMode: 'disabled' as AlarmModes };
    const result = validateTypeInsight(insight, true);
    expect(result).toBe(true);
  });

  it('should return false when conditionsCheck is empty in "conditional" alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'conditional' as AlarmModes, conditionsCheck: [] };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(false);
  });

  it('should return false when a condition is invalid in "conditional" alarmMode', () => {
    const insight = {
      ...baseInsight,
      alarmMode: 'conditional' as AlarmModes,
      conditionsCheck: [{ nodeLabel: '', statistic: 'average', operator: '>' as conditionalOperators, value: 10 }],
    };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(false);
  });

  it('should return true when all conditions are valid in "conditional" alarmMode', () => {
    const insight = {
      ...baseInsight,
      alarmMode: 'conditional' as AlarmModes,
      conditionsCheck: [{ nodeLabel: 'label', statistic: 'average', operator: '>' as conditionalOperators, value: 10 }],
    };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(true);
  });

  it('should return false when frequency or previousResultHash is missing in "diff" alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'diff' as AlarmModes, frequency: '', previousResultHash: '' };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(false);
  });

  it('should return true when frequency and previousResultHash are provided in "diff" alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'diff' as AlarmModes, frequency: 'daily', previousResultHash: 'hash' };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(true);
  });

  it('should return false when frequency is missing in "always" alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'always' as AlarmModes, frequency: '' };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(false);
  });

  it('should return true when frequency is provided in "always" alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'always' as AlarmModes, frequency: 'daily' };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(true);
  });

  it('should return true for unknown alarmMode', () => {
    const insight = { ...baseInsight, alarmMode: 'unknown' as any };
    const result = validateTypeInsight(insight, false);
    expect(result).toBe(true);
  });
});
